@@ -5,6 +5,7 @@ from __future__ import print_function
 import os
 import sys
 import pytest
+import subprocess
 ## 3rd party
 import numpy as np
 import pandas as pd
@@ -16,6 +17,15 @@ from MGSIM.Commands import Reads as Reads_CMD
 test_dir = os.path.join(os.path.dirname(__file__))
 data_dir = os.path.join(test_dir, 'data')
 
+def validate_fastq(data_dir, paired=True):
+    # validating fastq
+    output_R = os.path.join(data_dir, 'TEST', '1', 'R1.fq')
+    ret = subprocess.run(['fqtools', 'validate', output_R])
+    assert ret.returncode == 0
+    if paired is True:
+        output_R = os.path.join(data_dir, 'TEST', '1', 'R2.fq')
+        ret = subprocess.run(['fqtools', 'validate', output_R])
+        assert ret.returncode == 0
 
 # tests
 def test_help(script_runner):
@@ -34,6 +44,7 @@ def test_main(script_runner):
                             '--rndSeed', '8294',
                             genomeList, abund_table, output_prefix)
     assert ret.success
+    validate_fastq(data_dir)
 
 def test_main_multiProc(script_runner):
     genomeList = os.path.join(data_dir, 'genome_list.txt')
@@ -48,6 +59,7 @@ def test_main_multiProc(script_runner):
                             '--rndSeed', '8294',
                             genomeList, abund_table, output_prefix)
     assert ret.success
+    validate_fastq(data_dir)
 
 def test_main_unpaired(script_runner):
     genomeList = os.path.join(data_dir, 'genome_list.txt')
@@ -61,4 +73,5 @@ def test_main_unpaired(script_runner):
                             '--rndSeed', '8294',
                             genomeList, abund_table, output_prefix)
     assert ret.success
+    validate_fastq(data_dir, False)
 
